package com.jiuqi.sql.impl;

import com.jiuqi.sql.DBException;
import com.jiuqi.sql.FieldInfo;
import com.jiuqi.sql.InputRecordBuffer;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

public class DBaseInputRecordBuffer implements InputRecordBuffer {
   private char deleteFlag;
   private byte[] buffer;
   private boolean prevNull;
   private DBaseFileTail strValueList;
   private boolean supportText;

   public DBaseInputRecordBuffer(int recordSize, boolean supportText, DBaseFileTail strValueList) {
      this.buffer = new byte[recordSize];
      this.supportText = supportText;
      this.strValueList = strValueList;
   }

   public boolean readRecord(InputStream stream) throws DBException {
      boolean result = false;

      try {
         this.deleteFlag = (char)stream.read();
         if (!this.supportText || this.deleteFlag != 26) {
            int readSize = stream.read(this.buffer);
            result = readSize == this.buffer.length;
         }

         return result;
      } catch (IOException var5) {
         IOException ex = var5;
         DBException dbe = new DBException();
         dbe.setStackTrace(ex.getStackTrace());
         throw dbe;
      }
   }

   public boolean isDeleted() {
      return this.deleteFlag == '*';
   }

   private boolean isNull(FieldInfo info) {
      boolean flag;
      int empty;
      switch (info.type) {
         case 1:
            if (this.buffer[info.offset] == 32) {
               return true;
            }

            return false;
         case 2:
         case 8:
            flag = false;

            for(empty = 0; empty < 8; ++empty) {
               if (this.buffer[info.offset + empty] != 32) {
                  flag = true;
                  break;
               }
            }

            return !flag;
         case 3:
            flag = false;

            for(empty = 0; empty < 8; ++empty) {
               if (this.buffer[info.offset + empty] != 0) {
                  flag = true;
                  break;
               }
            }

            return !flag;
         case 4:
         case 7:
         default:
            return false;
         case 5:
            flag = false;

            for(empty = 0; empty < 4; ++empty) {
               if (this.buffer[info.offset + empty] != 0) {
                  flag = true;
                  break;
               }
            }

            return !flag;
         case 6:
            flag = false;
            empty = 0;
            int size = info.size;
            if (this.supportText) {
               empty = -1;
               size = 4;
            }

            for(int i = 0; i < size; ++i) {
               if (this.buffer[info.offset + i] != empty) {
                  flag = true;
                  break;
               }
            }

            return !flag;
      }
   }

   public boolean readBoolean(FieldInfo info) throws DBException {
      this.prevNull = this.isNull(info);
      return this.prevNull ? false : RW.readBoolean(this.buffer, info.offset);
   }

   public double readDouble(FieldInfo info) throws DBException {
      this.prevNull = this.isNull(info);
      return this.prevNull ? 0.0 : RW.readDouble(this.buffer, info.offset);
   }

   public int readInt(FieldInfo info) throws DBException {
      this.prevNull = this.isNull(info);
      return this.prevNull ? 0 : RW.readInt(this.buffer, info.offset);
   }

   public String readString(FieldInfo info) throws DBException {
      this.prevNull = this.isNull(info);
      if (this.prevNull) {
         return null;
      } else if (this.supportText) {
         int index = RW.readInt(this.buffer, info.offset);
         return this.strValueList.get(index);
      } else {
         return RW.readString(this.buffer, info.offset, info.size);
      }
   }

   public long readLong(FieldInfo info) throws DBException {
      this.prevNull = this.isNull(info);
      return this.prevNull ? 0L : RW.readLong(this.buffer, info.offset);
   }

   public Date readDate(FieldInfo info) throws DBException {
      this.prevNull = this.isNull(info);
      return this.prevNull ? null : RW.readDateTime(this.buffer, info.offset);
   }

   public boolean wasNull() throws DBException {
      return this.prevNull;
   }
}
